# Concept: templates

Templates in C++ are a powerful feature that allows you to write generic, reusable code. Templates enable functions and classes to operate with any data type without rewriting the same code for each type. There are two main types of templates in C++: function templates and class templates.

## Key Characteristics of Templates

- Generic Programming: Templates support generic programming, allowing you to write code that works with any data type.
- Type Safety: Templates provide type safety, as the compiler checks that the correct types are used at compile time.
- Code Reusability: Templates enable you to reuse code across different data types, reducing duplication and potential errors.
- Template Instantiation: When a template is used, the compiler generates a specific instance of the template for the given data type.

## Function Templates

Function templates allow you to write a single function that works with different data types. The template is defined with a placeholder type, which is replaced with the actual data type when the function is called.

## Example 1: Simple Function Template

```cpp
#include <iostream>

// Define a function template that adds two values
template<typename T>
T add(T a, T b) {
    return a + b;
}

int main() {
    int x = 5, y = 10;
    double a = 2.5, b = 3.7;

    // Call the function template with different types
    std::cout << "Sum of integers: " << add(x, y) << std::endl;
    std::cout << "Sum of doubles: " << add(a, b) << std::endl;

    return 0;
}
```

### Explanation:

- Template Definition: The add function template is defined with the template<typename T> syntax, where T is a placeholder for the data type.
- Template Instantiation: When add(x, y) is called, the compiler generates a specific version of the add function for int. Similarly, add(a, b) generates a version for double.

## Example 2: Function Template with Multiple Types

You can define a function template that works with multiple data types by specifying more than one template parameter.

```cpp
#include <iostream>

// Define a function template that compares two values of different types
template<typename T1, typename T2>
bool areEqual(T1 a, T2 b) {
    return a == b;
}

int main() {
    int x = 5;
    double y = 5.0;

    // Call the function template with different types
    std::cout << "Are they equal? " << (areEqual(x, y) ? "Yes" : "No") << std::endl;

    return 0;
}
```

### Explanation:

- Multiple Template Parameters: The areEqual function template is defined with two template parameters, T1 and T2, allowing it to compare values of different types.
- Type Deduction: The compiler automatically deduces the types of T1 and T2 based on the arguments passed to the function.

## Example 3: Class Templates

Class templates allow you to create classes that work with any data type. The class is defined with one or more placeholder types, which are specified when an object of the class is created.

```cpp
#include <iostream>

// Define a class template
template<typename T>
class Box {
private:
    T value;
public:
    Box(T val) : value(val) {}

    T getValue() const {
        return value;
    }
};

int main() {
    Box<int> intBox(123);
    Box<double> doubleBox(45.67);

    // Access the value stored in the boxes
    std::cout << "Integer Box: " << intBox.getValue() << std::endl;
    std::cout << "Double Box: " << doubleBox.getValue() << std::endl;

    return 0;
}
```

### Explanation:

- Template Definition: The Box class template is defined with template<typename T>, where T is a placeholder for the data type.
- Instantiation: When Box<int> and Box<double> objects are created, the compiler generates specific versions of the Box class for int and double, respectively.

## Example 4: Class Template with Multiple Types

Class templates can also work with multiple data types by specifying more than one template parameter.

```cpp
Copy code
#include <iostream>

// Define a class template with two types
template<typename T1, typename T2>
class Pair {
private:
    T1 first;
    T2 second;
public:
    Pair(T1 a, T2 b) : first(a), second(b) {}

    T1 getFirst() const {
        return first;
    }

    T2 getSecond() const {
        return second;
    }
};

int main() {
    Pair<int, double> p(42, 3.14);

    std::cout << "First: " << p.getFirst() << std::endl;
    std::cout << "Second: " << p.getSecond() << std::endl;

    return 0;
}
```

### Explanation:

- Multiple Template Parameters: The Pair class template is defined with two template parameters, T1 and T2.
- Instantiation: The Pair<int, double> object is created, and the compiler generates a version of the Pair class for int and double.
- Specialization of Templates
- Sometimes, you may want to define a specialized version of a template for a specific data type. This is known as template specialization.

## Example 5: Function Template Specialization

```cpp
Copy code
#include <iostream>

// Generic template
template<typename T>
T getMax(T a, T b) {
    return (a > b) ? a : b;
}

// Specialized template for const char*
template<>
const char* getMax(const char* a, const char* b) {
    return (std::strcmp(a, b) > 0) ? a : b;
}

int main() {
    std::cout << "Max of 10 and 20: " << getMax(10, 20) << std::endl;
    std::cout << "Max of 'Apple' and 'Banana': " << getMax("Apple", "Banana") << std::endl;

    return 0;
}
```

### Explanation:

- Generic Template: The getMax function template works with any data type.
- Specialization: A specialized version of the getMax template is defined for const char* to compare C-style strings.

## Example 6: Class Template Partial Specialization

You can also partially specialize a class template for certain types.

```cpp
#include <iostream>

// Generic template
template<typename T, typename U>
class MyClass {
public:
    void show() {
        std::cout << "Generic MyClass" << std::endl;
    }
};

// Partial specialization for T = int
template<typename U>
class MyClass<int, U> {
public:
    void show() {
        std::cout << "Specialized MyClass for int" << std::endl;
    }
};

int main() {
    MyClass<double, double> obj1;
    obj1.show();  // Outputs: Generic MyClass

    MyClass<int, double> obj2;
    obj2.show();  // Outputs: Specialized MyClass for int

    return 0;
}
```

### Explanation:

- Generic Template: The MyClass template is defined for two types T and U.
- Partial Specialization: A partially specialized version of MyClass is defined for T = int, which changes the behavior of the show method when T is int.
- Template Metaprogramming

Templates in C++ can also be used for template metaprogramming, a technique where templates are used to perform computations at compile time. This is an advanced usage of templates that can lead to highly efficient and optimized code.

## Summary

- Templates: A feature in C++ that allows you to write generic code that works with any data type. Templates increase code reusability and reduce duplication.
- Function Templates: Allow you to define functions that work with different types, with the compiler generating specific instances based on the types used.
- Class Templates: Enable the creation of classes that can work with any data type, instantiated with the specific type when an object is created.
- Specialization: You can specialize templates for specific types or cases, providing custom behavior when necessary.
- Template Metaprogramming: An advanced use of templates that allows computations to be performed at compile time, optimizing the generated code.

Templates are a fundamental feature of C++, enabling generic programming and allowing for highly reusable and efficient code. Understanding templates is essential for mastering C++ and writing flexible, type-safe code.
